"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestClient = void 0;
const querystring = require("querystring");
const fetchPonyfill = require("fetch-ponyfill");
const { fetch } = fetchPonyfill();
const toQueryString = (obj) => {
    if (!Object.keys(obj).length) {
        return '';
    }
    return '?' + querystring.stringify(obj);
};
class RestClient {
    constructor() {
        this.headers = {
            Accept: 'application/json',
        };
        this.endpoint = '';
        this.response = {
            headers: {},
            statusCode: -1,
            body: '',
        };
    }
    async request(method, path, queryString, extraHeaders, body, debug) {
        var _a, _b;
        const headers = {
            ...this.headers,
            ...extraHeaders,
        };
        const url = `${this.endpoint.replace(/\/+$/, '')}/${path.replace(/^\/+/, '')}${toQueryString(queryString !== null && queryString !== void 0 ? queryString : {})}`;
        const res = await fetch(url, {
            method,
            headers,
            body: body !== undefined
                ? typeof body !== 'string'
                    ? JSON.stringify(body)
                    : body
                : undefined,
        });
        const contentType = (_a = res.headers.get('content-type')) !== null && _a !== void 0 ? _a : '', mediaType = contentType.split(';')[0];
        if (!headers.Accept.includes(mediaType)) {
            if (debug !== undefined)
                debug(`[REST]`, JSON.stringify({ headers, body: await res.text() }));
            throw new Error(`The content-type "${contentType}" of the response does not match accepted media-type ${headers.Accept}`);
        }
        const statusCode = res.status;
        const contentLength = +((_b = res.headers.get('content-length')) !== null && _b !== void 0 ? _b : 0);
        const h = {};
        res.headers.forEach((v, k) => {
            h[k] = v;
        });
        if (contentLength > 0 &&
            /^application\/([^ /]+\+)?json$/.test(mediaType) === false) {
            if (debug !== undefined)
                debug(`[REST]`, JSON.stringify({ headers, body: await res.text() }));
            throw new Error(`The content-type "${contentType}" of the response is not JSON!`);
        }
        this.response = {
            statusCode,
            headers: h,
            body: contentLength ? await res.json() : undefined,
        };
        return url;
    }
}
exports.RestClient = RestClient;
